package cat.hack3.codingtests.marsrover;

import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.function.IntConsumer;
import java.util.stream.IntStream;

import static cat.hack3.codingtests.marsrover.MarsMap.Direction;
import static cat.hack3.codingtests.marsrover.MarsMap.Direction.*;
import static org.testng.Assert.assertEquals;

public class MarsRoverTest {

    private MarsMap marsMap;
    private MarsRover rover;

    @BeforeMethod
    public void setUp() {
        int mapWidth = 10;
        int mapHeight = 10;
        marsMap = new MarsMap(mapWidth, mapHeight);

        int latitudeStartingPoint = 2;
        int longitudeStartingPoint = 3;
        var startingCoordinates = Coordinates.of(latitudeStartingPoint, longitudeStartingPoint);
        rover = new MarsRover(marsMap, startingCoordinates, SOUTH);
    }

    @Test
    public void roverMakeItsFirstStep() {
        Coordinates currentPosition = rover.moveForward();
        assertEquals(currentPosition, Coordinates.of(3, 3));
        assertEquals(rover.getCurrentDirection(), SOUTH);
    }

    @Test
    public void make5StepsInARow() {
        repeatAction(5, i -> rover.moveForward());
        assertEquals(rover.getCurrentCoordinates(), Coordinates.of(8, 3));
        assertEquals(rover.getCurrentDirection(), SOUTH);
    }

    @Test
    public void turnLef() {
        Direction currentDirection = rover.turnLeft();
        assertEquals(currentDirection, EAST);
        assertEquals(rover.turnLeft(), NORTH);
        assertEquals(rover.turnLeft(), WEST);
        repeatAction(2, i -> rover.turnLeft());
        assertEquals(currentDirection, EAST);
    }

    private void repeatAction(int times, IntConsumer actionToRepeat) {
        IntStream.rangeClosed(0, times)
                .forEach(actionToRepeat);
    }
}
